#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "forkawarelocal4xi.py">
#<abstract4xi abstract = "provides the class ForkAwareLocal4Xi"/>
#<refer4xi reference = "util.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'ForkAwareLocal4Xi' ]
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # most-common ones
    import itertools
    import weakref
    import atexit         
except:
    pass
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi 
from reason_object4xi import reason_object4xi
from register_after_fork4xi import register_after_fork4xi
from print4xi import print4xi
from sense_object4xi import sense_object4xi
from time4xi import time4xi
#"
#/>

# =================================================================================================
#<class4xi class = "ForkAwareLocal4Xi" base = "threading.local" child = "None" synopsis = "fork aware local" usage = "
#     managersi.py:
#         tls_idset = ForkAwareLocal(), ProcessLocalSet(), where tls = thread local storage
#     process.py:
#         tls_idset = ForkAwareLocal(), ProcessLocalSet()
#
# baseproxy4xi.py:
#     tls_idset = ForkAwareLocal4Xi(), ProcessLocalSet4Xi()"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
class ForkAwareLocal4Xi(threading.local):
    '''
    ForkAwareLocal4Xi class: in multiprocessing managersi.py/process.py: tls_idset = ForkAwareLocal(), ProcessLocalSet()
    
    '''
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi exposure = "
    _exposed_ = ('sense', 'reason', 'ace')          
    #"
    #/>
          
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-ForkAwareLocal4Xi" usage = "initializes ForkAwareLocal4Xi">                  
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self):

        # -----------------------------------------------------------------------------------------
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------
        
        class2print = "ForkAwareLocal4Xi"
        method2print = "__init__"

        message2print = "{(Step 1/3a) started" + "; init-of-ForkAwareLocal4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                        
        
        message2print = "{(Step 1/3b) borning" + "; forkawarelocal2xi-of-ForkAwareLocal4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             

        message2print = "{(Step 1/3c) inheriting" + "; threading-local-inheriting-by-forkawarelocal2xi-of-ForkAwareLocal4Xi: " + str(threading.local)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                        
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------
                     
        message2print = "{(Step 2/3a) calling" + "; register-after-fork4xi-calling-by-init-of-ForkAwareLocal4Xi: " + str(register_after_fork4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
                        
        message2print = "{(Step 2/3b) passing" + "; dict-passing-to-register-after-fork4xi-by-init-of-ForkAwareLocal4Xi: " + str(lambda obj : obj.__dict__.clear()) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                        
        try:
            register_after_fork4xi (self, lambda obj : obj.__dict__.clear())
        
            message2print = "{(Step 2/3d) performed" + "; register-after-fork4xi-performed-by-init-of-ForkAwareLocal4Xi: " + str(register_after_fork4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
        except:
            message2print = "{(Step 2/3e) hit" + "; register-after-fork4xi-hit-by-init-of-ForkAwareLocal4Xi: " + " *** " + str(register_after_fork4xi)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
    
            raise
        # -----------------------------------------------------------------------------------------
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3/3a) inherited" + "; threading-local-inherited-by-forkawarelocal2xi-of-ForkAwareLocal4Xi: " + str(threading.local)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                              
        message2print = "{(Step 3/3b) activated" + "; forkawarelocal2xi-of-ForkAwareLocal4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  

        message2print = "{(Step 3/3c) returning" + "; init-of-ForkAwareLocal4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                         
     

    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reduce-of-ForkAwareLocal4Xi" usage = "(self)">                        
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------    
    def __reduce__(self):

        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------    
        class2print = "ForkAwareLocal4Xi"
        method2print = "__reduce__"

        message2print = "{started" + "; reduce-of-ForkAwareLocal4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
          
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
        message2print = "{returning" + "; type-of-forkawarelocal2xi-returning-by-reduce-of-ForkAwareLocal4Xi: " + str(type(self)) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                        
        return type(self), ()
    #</methodize4xi>
                                           
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-ForkAwareLocal4Xi" usage = "senses ForkAwareLocal4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses ForkAwareLocal4Xi class objects
        '''
        
        
        class2print = 'ForkAwareLocal4Xi'
        method2print = 'sense'
        
        message2print = "{started" + "; sense-of-ForkAwareLocal4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; ForkAwareLocal4Xi-sensing-by-sense-of-ForkAwareLocal4Xi: " + str(ForkAwareLocal4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                      
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-ForkAwareLocal4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
    
        
        message2print = "{sensed" + "; ForkAwareLocal4Xi-sensed-by-sense-of-ForkAwareLocal4Xi: " + str(ForkAwareLocal4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-ForkAwareLocal4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                          
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-ForkAwareLocal4Xi" usage = "reasons ForkAwareLocal4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons ForkAwareLocal4Xi class objects
        '''
         
        
        class2print = 'ForkAwareLocal4Xi'
        method2print = 'reason'

        message2print = "{started" + "; reason-of-ForkAwareLocal4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
    
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; ForkAwareLocal4Xi-reasoning-by-reason-of-ForkAwareLocal4Xi: " + str(ForkAwareLocal4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-ForkAwareLocal4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
        message2print = "{reasoned" + "; ForkAwareLocal4Xi-reasoned-by-reason-of-ForkAwareLocal4Xi: " + str(ForkAwareLocal4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-ForkAwareLocal4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                        
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-ForkAwareLocal4Xi" usage = "aces ForkAwareLocal4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces ForkAwareLocal4Xi class objects 
        '''
               
        
        
        class2print = 'ForkAwareLocal4Xi'
        method2print = 'ace'
        
        message2print = "{started" + "; ace-of-ForkAwareLocal4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; ForkAwareLocal4Xi-acing-by-ace-of-ForkAwareLocal4Xi: " + str(ForkAwareLocal4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                      
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-ForkAwareLocal4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; ForkAwareLocal4Xi-aced-by-ace-of-ForkAwareLocal4Xi: " + str(ForkAwareLocal4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-ForkAwareLocal4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                              
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of forkawarelocal4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>          
